package com.gihon.security.service.impl;

import com.gihon.user.mapper.UserMapper;
import com.gihon.user.entity.User;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 自定义登录逻辑实现。
 */
@Service
public class UserLoginServiceImpl implements UserDetailsService {
    @Resource
    private UserMapper userMapper;

    // 登录逻辑，访问数据库实现。
    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        System.out.println("开始登陆  验证逻辑");
        // 1、 访问数据库，根据用户名查询用户。
        User user = userMapper.selectByUsername(username);

        // 2、 判断查询结果是否存在， 决定是否需要抛出异常。
        if(null == user){
            // 用户不存在，抛出异常。
            throw new UsernameNotFoundException("用户不存在");
        }

        // 3、 查询认证用户的权限集合。并封装成SpringSecurity需要的集合。List<GrantedAuthority>
        // 用户的权限包括，角色和权限， 就是role和permission
        // SpringSecurity中，要求角色的字符串描述信息，必须使用"ROLE_"作为前缀
        // SpringSecurity中，对权限的字符串描述信息，没有强制要求。
        // 3.1 查询登录用户的角色名集合
        List<String> roleNames = userMapper.selectRoleNamesByUserId(user.getId());
        // 3.2 查询登录用户的权限描述集合。就是tb_permission表格中permission字段的集合。
        List<String> permissions = userMapper.selectPermissionsByUserId(user.getId());
        // 3.3 封装一个List<GrantedAuthority>类型的集合
        List<GrantedAuthority> grantedAuthorities = new ArrayList<>(roleNames.size()+permissions.size());
        // 把角色保存到权限集合中
        for(String roleName : roleNames){
            // 角色字符串描述必须使用"ROLE_"开头
            roleName = "ROLE_" + roleName;
            grantedAuthorities.add(new SimpleGrantedAuthority(roleName));
        }
        for(String perm : permissions){
            // 权限的集合没有特殊要求
            grantedAuthorities.add(new SimpleGrantedAuthority(perm));
        }

        // 4、 返回UserDetails接口的实现对象， SpringSecurity提供的User类型。

        return  new org.springframework.security.core.userdetails.User(
                username, user.getPassword(),
                grantedAuthorities  // 提供当前登录用户的权限集合，其中包括角色描述和权限描述。
        );
    }



}
